/**
 * A simple graphical and textual Javascript clock.
 *
 * Loads and instantiates a graphical or textual version of a highly customizable Javascript clock.
 * All important aspects of the clock can be customized including the graphical images sets that will
 * be used to represent the clock when running in graphical mode.
 *
 * @package          Janus
 * @subpackage       Javascript       
 * @category         Clocks
 * @author           Misa Brezanac <brezanac@gmail.com>
 * @license GNU General Public License (http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt)
*/
 
/**
 * Controls if the clock should automatically start after loading.
*/
var AUTOSTART = true;

/**
 * Controls the clocks running mode (textual or graphical).
 * If running in graphical mode the clock makes use of the IMAGES options supplied bellow.
 * Otherwise a typical textual representation of the clock is generated.
*/
var USE_IMAGES = true;

/**
 * Should the clock display seconds?
*/
var DISPLAY_SECONDS = false;

/**
 * Path to the clock images set to be used when generating output (WITH THE TRAILING SLASH)
*/
var IMAGES_PATH = "images/clock/black/";

/**
 * Should the clock images be preloaded?
*/
var IMAGES_PRELOAD = true;

/**
 * Image files prefix
*/
var IMAGES_PREFIX = "";

/**
 * Image files filetype (WITH THE PRECEEDING DOT!)
*/
var IMAGES_FILETYPE = ".png";

/**
 * Name of the 'blank' image (WITHOUT PREFIX AND FILETYPE)
*/
var IMAGES_BLANK = "blank";

/**
 * The name of the 'OFF' separator image (WITHOUT PREFIX AND FILETYPE)
*/
var IMAGES_SEPARATOR_OFF = "sep";

/**
 * The name of the 'ON' separator image (WITHOUT PREFIX AND FILETYPE)
*/
var IMAGES_SEPARATOR_ON = "sepa";

/**
 * The default CSS id selector of the DIV container that will hold the clocks generated HTML
*/
var CLOCK_ID = "clockContainer";

/**
 * A weekday names array. Must begin with "Sunday".
*/
var weekdays = new Array("Nedelja", "Ponedeljak", "Utorak", "Sreda", "Cetvrtak", "Petak", "Subota");

/**
 * Clocks cycle refresh interval in milliseconds. Not particularly useful except
 * for debugging or in case you know exactly why you don't want to make your clock refresh every second.
*/
var CLOCK_REFRESH_INTERVAL = 1000;

/**                                                                                    
 * Since Javascript OOP concepts suck, to say the least, the preloaded images arrays are declared 
 * globally. Consider them as publically declared arrays :) 
 * Images are only preloaded if the graphical mode is used.
*/
if((USE_IMAGES == true) && (IMAGES_PRELOAD == true)){
    /**
     * Storing all of the clock digit images in the array
    */
    this.imagesArray = new Array();
    for(i = 0; i <= 9; i++){
        this.imagesArray[i] = new Image();
        this.imagesArray[i].src = IMAGES_PATH + IMAGES_PREFIX + i + IMAGES_FILETYPE;
    }
    
    /**
     * Storing the two additional separator images
    */
    this.imagesSeparators = new Array(new Image(), new Image());
    this.imagesSeparators[0].src = IMAGES_PATH + IMAGES_PREFIX + IMAGES_SEPARATOR_OFF + IMAGES_FILETYPE; //the inactive separator image
    this.imagesSeparators[1].src = IMAGES_PATH + IMAGES_PREFIX + IMAGES_SEPARATOR_ON + IMAGES_FILETYPE; //the active separator image
}

/**
 * Clock class constructor. Did I mention that constructors suck too in Javascript? :) 
*/
function Clock(){ 
    /**
     * Assigning functions as objects methods to the object.
     * Well methods (functions to be more precise) are the actual winners when it comes to NOT holding up with OOP concepts 
     * and benefits
     * that OOP is supposed to provide? Why... I'll just say that object methods are 'executed' in a
     * contextual manner meaning that what a method does depends upon the actual object calling for it.
    */
    this.prepareClock = prepareClock;
    this.show = show;
    this.hide = hide;
    this.start = start;
    this.stop = stop;
    this.doClock = doClock;
    
    /**
     * If the clocks HTML is not yet been initialized we need to use Javascript to initialize required objects
     * in case a image preload mode is used.
    */
    this.prepareClock();
    
    /**
     * Is the AUTOSTART function set to true? If yes the clock will begin with its work imediatelly.
    */
    if(AUTOSTART == true){
        this.start();
    }
}

/**
 * Clock initialization. This method will simply populate the DIV container specified in the CLASS_ID variable with
 * the required HTML code that is going to be manipulated by Javascript.
*/
function prepareClock(){
    this.outputHTML = '<div id="mainClock">'; //the container that will actually hold the generated digits
    this.initIdArray = new Array('hours1', 'hours0', 'sep', 'minutes1', 'minutes0', 'sep2', 'seconds1', 'seconds0');
    for(i = 0; i < (DISPLAY_SECONDS == true ? 8 : 5); i++){
        this.outputHTML += '<img id="' + this.initIdArray[i].toString() + '" name="' + this.initIdArray[i].toString() + '" src="" />';   
    }
    
    /**
     * Adding some required containers and populating the DOM tree with the generated info.
    */
    this.outputHTML += '</div><div id="dateContainer"><div id="weekday"></div><div id="date"></div>';
    $(CLOCK_ID).innerHTML = this.outputHTML;
}

/**
 * Displays the clock by altering it's containers display CSS property. 
 * Used to cover the image loading delay at clock loading time.
*/
function show(){
    $("mainClock").style.display = "block";
}

/**
 * Hides the clock by altering it's containers display CSS property. 
 * Not actually used in this script but you never know when you might need it ;)
*/
function hide(){
    $("mainClock").style.display = "none";
}

/**
 * If I need to explain to you what this method does than you might consider choosing some other job to
 * sustain your existence on this world other than web programming :) No... Just kidding... This method simply starts the clock. 
*/
function start(){
    /**
     * If there is a previous instance of a clock interval we need to destroy it prior 
     * to initializing a new one.
    */
    if(this.clockInterval != null){
        this.stop();    
    }
    this.clockInterval = setInterval('this.doClock()', CLOCK_REFRESH_INTERVAL);
}

/**
 * Gee... If the previous method is used to start the clock I wonder what this one might be doing :)\
 * Yes it does... It is designed to stop the clock.
*/
function stop(){
    clearInterval(this.clockInterval);
}

/**
 * Executes a new clock calculation and display cycle. This is the method that actually does most of the work.
*/
function doClock(){
    /**
     * First we need to get some information about the current time and date.
    */
    this.currentDate = new Date();
    this.dayOfWeek = this.currentDate.getDay();
    this.day = this.currentDate.getDate();
    this.month = this.currentDate.getMonth();
    this.year = this.currentDate.getFullYear();
    this.hours = this.currentDate.getHours().toString();
    this.minutes = this.currentDate.getMinutes().toString();
    this.seconds = this.currentDate.getSeconds().toString();
    //delete this.currentDate; //explicitely releasing the Date object's occupied memory (Not working in IE6!!!)
    
    /**
     * Taking care about the fact that hours also have a one digit state (0-9).
    */
    if(this.hours <= 9){
        this.hours1 = "0";
        this.hours0 = this.hours.charAt(0);
    } else {
        this.hours1 = this.hours.charAt(0);
        this.hours0 = this.hours.charAt(1);
    }
    
    /**
     * Minutes too have a one digit state (0-9)
    */
    if(this.minutes <= 9){
        this.minutes1 = "0";
        this.minutes0 = this.minutes.charAt(0);
    } else {
        this.minutes1 = this.minutes.charAt(0);    
        this.minutes0 = this.minutes.charAt(1); 
    }
    
    /**
     * Oh boy... Seconds also have a one digit state (0-9), right? :)
    */
    if(this.seconds <= 9){
        this.seconds1 = "0";
        this.seconds0 = this.seconds.charAt(0);
    } else {
        this.seconds1 = this.seconds.charAt(0);    
        this.seconds0 = this.seconds.charAt(1); 
    }
    
    /**
     * How about the separator? Is it active for the current second?
     * We are going to make the script render the inactive separator for every odd second
     * and the active one for every even second in the graphical mode. In textual mode a semicolon
     * is used to separate digit groups.
    */
    if((this.seconds % 2) == 0){
        this.separator = ":";
    } else {
        this.separator = " ";
    }
    
    /**
     * If this is the first time the script is displaying output then we need to first make the required container visible.
     * That would be the one that was hidden with CSS 'display' property.
    */
    this.show();
    
    /**
     * Generating the required HTML code inside the corresponding clock containers. 
     * If a preloader option is supplied then the preloaded images are used otherwise images are loaded as required.
    */
    if(USE_IMAGES){
        if(IMAGES_PRELOAD == true){
            $("hours1").src = this.imagesArray[parseInt(this.hours1)].src;
            $("hours0").src = this.imagesArray[parseInt(this.hours0)].src;
            
            /**
             * In case seconds are displayed too the first separator is not supposed to blink 
             * (hence it stays always active).
            */
            if(DISPLAY_SECONDS == true){
                $("sep").src = this.imagesSeparators[1].src;    
            } else {
                $("sep").src = (this.separator == ":" ? this.imagesSeparators[1] : this.imagesSeparators[0]).src;       
            }
            $("minutes1").src = this.imagesArray[parseInt(this.minutes1)].src;
            $("minutes0").src = this.imagesArray[parseInt(this.minutes0)].src;
            
            /**
             * The following block of code is executed only if second are displayed too
            */
            if(DISPLAY_SECONDS == true){
                $("sep2").src = this.imagesSeparators[1].src;
                $("seconds1").src = this.imagesArray[parseInt(this.seconds1)].src;
                $("seconds0").src = this.imagesArray[parseInt(this.seconds0)].src; 
            }    
        } else {
            this.outputHTML = '<div id="mainClock">';
            this.outputHTML = '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.hours1 + IMAGES_FILETYPE + '"/ >';
            this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.hours0 + IMAGES_FILETYPE + '"/ >';
            
            /**
             * Should the seconds be displayed in the non image preload mode?
            */
            if(DISPLAY_SECONDS == true){
                this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + IMAGES_SEPARATOR_ON + IMAGES_FILETYPE + '"/ >';    
            } else {
                this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + (this.separator == ":" ? IMAGES_SEPARATOR_ON : IMAGES_SEPARATOR_OFF) + IMAGES_FILETYPE + '"/ >';       
            }
            this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.minutes1 + IMAGES_FILETYPE + '"/ >';
            this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.minutes0 + IMAGES_FILETYPE + '"/ >';
            
            /**
             * If seconds are to be displayed in the non image preload mode then this is the place where its done
            */
            if(DISPLAY_SECONDS == true){
                 this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + IMAGES_SEPARATOR_ON + IMAGES_FILETYPE + '"/ >';    
                 this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.seconds1 + IMAGES_FILETYPE + '"/ >';
                 this.outputHTML += '<img src="' + IMAGES_PATH + IMAGES_PREFIX + this.seconds0 + IMAGES_FILETYPE + '"/ >';
            }
            this.outputHTML += '</div><div id="dateContainer"><div id="weekday"></div><div id="date"></div>';
            
            /**
             * Populating the DIV container in the non image preload mode.
            */
             $(CLOCK_ID).innerHTML = this.outputHTML;
        }
    } else {
         this.outputHTML = '<div id="mainClock">';
         this.outputHTML = this.hours1 + this.hours0 + ":" + this.minutes1 + this.minutes0;
         if(DISPLAY_SECONDS == true){
            this.outputHTML += ":" + this.seconds1 + this.seconds0;
         }
         this.outputHTML += '</div><div id="dateContainer"><div id="weekday"></div><div id="date"></div>';
         $(CLOCK_ID).innerHTML = this.outputHTML;
    }
    
    /**
     * Time to update the weekday and date containers with some info
    */
    $("weekday").innerHTML = weekdays[this.dayOfWeek];
    $("date").innerHTML = this.day + "." + (this.month + 1).toString() + "." + this.year + "."; 
    
}

/**
 * An id selector fetcher function.
 *
 * This functions main purpose is to return a reference to the object specified as the functions argument.
 * Take note that some Javascript frameworks like to reserve the dollar sign '$' for the same purpose it is used in 
 * this script, namely selecting HTML elements through ids ('jQuery' and 'prototype' for instance). 
 * Therefore if you need to please alter the code in this script and rename the '$' method to something meaningful.
 * After that please update the code in this script to reflect your changes. In other words update all of the existing
 * references to the '$' function with the new name.
 *
 * @param string id The name of the id selector to be referenced.
 * @return obj The reference to the requested object (id). On failure returns 'null'.
*/
function $(id){
    return document.getElementById(id);
}                       

/**
 * This function might seem like an overkill but its required for some browsers in order to correctly inteprete the 
 * window.onload event handler bellow. Oh yeah... It is simply used to instantiate a clock object while taking care 
 * that all of the required HTML elements are first generated with Javascript. 
*/
function initClock(){
    /**
     * The actual Clock object instantiation
    */
    var mainClock = new Clock();
}

/**
 * Used to ensure that everything in this script happens after the complete page is loaded.
*/
window.onload = initClock;
